# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

import re
import sys
from pathlib import Path

sys.path.append(str(Path("_extension").resolve()))


# get version number to print in docs
def get_version_info(filepath):
    with open(filepath, "r") as f:
        content = f.read()

    version_pattern = (
        r"^#define\s+AMDSMI_LIB_VERSION_MAJOR\s+(\d+)\s*$|"
        r"^#define\s+AMDSMI_LIB_VERSION_MINOR\s+(\d+)\s*$|"
        r"^#define\s+AMDSMI_LIB_VERSION_RELEASE\s+(\d+)\s*$"
    )

    matches = re.findall(version_pattern, content, re.MULTILINE)

    if len(matches) == 3:
        version_major, version_minor, version_release = [
            match for match in matches if any(match)
        ]
        return version_major[0], version_minor[1], version_release[2]
    else:
        raise ValueError("Couldn't find all VERSION numbers.")


version_major, version_minor, version_release = get_version_info(
    "../include/amd_smi/amdsmi.h"
)
version_number = f"{version_major}.{version_minor}.{version_release}"

# project info
project = "AMD SMI"
author = "Advanced Micro Devices, Inc."
copyright = "Copyright (c) 2025 Advanced Micro Devices, Inc. All rights reserved."
version = version_number
release = version_number

html_theme = "rocm_docs_theme"
html_theme_options = {"flavor": "rocm"}
html_title = f"AMD SMI {version_number} documentation"
suppress_warnings = ["etoc.toctree"]
external_toc_path = "./sphinx/_toc.yml"

external_projects_current_project = "amdsmi"
extensions = ["rocm_docs", "rocm_docs.doxygen", "go_api_ref"]

doxygen_root = "doxygen"
doxysphinx_enabled = True
doxygen_project = {
    "name": "AMD SMI C++ API reference",
    "path": "doxygen/docBin/xml",
}


def generate_doxyfile(app, _):
    doxyfile_in = Path(app.confdir) / doxygen_root / "Doxyfile.in"
    doxyfile_out = Path(app.confdir) / doxygen_root / "Doxyfile"

    if not doxyfile_in.exists():
        from sphinx.errors import ConfigError

        raise ConfigError(f"Missing Doxyfile.in at {doxyfile_in}")

    with open(doxyfile_in) as f:
        content = f.read()

    content = content.replace("@PROJECT_NUMBER@", version_number)

    with open(doxyfile_out, "w") as f:
        f.write(content)


def setup(app):
    app.connect("config-inited", generate_doxyfile, priority=100)
    return {"parallel_read_safe": True, "parallel_write_safe": True}



import re
import argparse

def get_platforms(file_name):
  # removing the new line characters
  with open(file_name) as f:
      lines = [line.rstrip() for line in f]
  
  platform_map = {}
  platform_lines = ""
  function_line = ""
  for line in lines:
      if "@platform" in line:
          platform_lines += line.replace("*","").lstrip()
      if platform_lines =="":
          continue
      l = line.lstrip()
      if len(l) == 0:
          continue
      if l[0] == '*' or l[0]== '#' or (len(l)>1 and l[0] == '/' and l[1] == '*'):
          if function_line != "":
              func = ' '.join(function_line.split())
              platforms = re.findall(r"\{(.*?)\}", platform_lines)
              for p in platforms:
                  if p not in platform_map:
                      platform_map[p]=[func]
                  else:
                      platform_map[p].append(func)
              platform_lines = ""
          function_line = ""
          continue
      function_line += line  + " " 
  return platform_map

# string in list1 but not in list2
def diff(list1, list2):
    result = []
    for l1 in list1:
        if l1 not in list2:
            result.append(l1)
    return result

def common(list1, list2):
    result = []
    for l1 in list1:
        if l1 in list2:
            result.append(l1)
    return result

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='amdsmi platform classifier')
    parser.add_argument('--platforms', default=False, action='store_true', help='list supported platforms')
    parser.add_argument('--list', default="", metavar="platform" , help='List function in a platform')
    parser.add_argument('--diff' , default=None, metavar="platform", nargs=2, help='Find the APIs in platform1 but not in platform2')
    parser.add_argument('--common' , default=None, metavar="platform", nargs=2, help='Find the common APIs of two platforms')

    file_name="../include/amd_smi/amdsmi.h"
    args = parser.parse_args()
    platform_map=get_platforms(file_name)
    if args.platforms:
        for p in platform_map:
            print (p)
        exit(0)

    if args.list:
        if args.list not in platform_map:
            print("Unknown platform ", args.list)
            exit(1)
        for f in platform_map[args.list]:
            print (f)
        exit(0)
        
    if args.diff != None:
        if args.diff[0] not in platform_map or args.diff[1] not in platform_map:
            print("Unknown platforms ", args.diff)
            exit(1)
        platforms0 = platform_map[args.diff[0]]
        platforms1 = platform_map[args.diff[1]]
        result = diff(platforms0, platforms1)
        print("APIs in", args.diff[0], "but not in", args.diff[1])
        for f in result:
            print(f)
        exit(0)
        
        
    if args.common != None:
        if args.common[0] not in platform_map or args.common[1] not in platform_map:
            print("Unknown platforms ", args.common)
            exit(1)
        platforms0 = platform_map[args.common[0]]
        platforms1 = platform_map[args.common[1]]
        result = common(platforms0, platforms1)
        print("APIs in both ", args.common[0], "and", args.common[1])
        for f in result:
            print(f)
        exit(0)
        
    parser.print_help()

